# Apache2 Log Formats (see: https://httpd.apache.org/docs/2.4/mod/mod_log_config.html#formats)
# LogFormat "%v:%p %h %l %u %t \"%r\" %>s %O \"%{Referer}i\" \"%{User-Agent}i\" %{forensic-id}n" vhost_combined
# We are using the 'combined' log.
# LogFormat "%h %l %u %t \"%r\" %>s %O \"%{Referer}i\" \"%{User-Agent}i\" %{forensic-id}n" combined
# LogFormat "%h %l %u %t \"%r\" %>s %O" common
# LogFormat "%{Referer}i -> %U" referer
# LogFormat "%{User-agent}i" agent

import os
import re
import sys
import mysql.connector

import L2Dcfg

def pause_until_key_press():
    print("Press Enter to continue...")
    text = input()
    if 'q' in text: return True
    else: return False

def process_line(line):
    # This needs to be dynamically detected when we open the file. Check to see if
    # forensic id is present in the log when opening.
    forensic_used = True

    # Processing starts here.
    log_line = line  # Store the line in a variable. May get rid of this later.

    # Parse the IP address field. ----------
    # Remote hostname. Will log the IP address if HostnameLookups is set to Off,
    # which is the default. If it logs the hostname for only a few hosts, you
    # probably have access control directives mentioning them by name. See the
    # Require host documentation.
    line_index = log_line.find(' ')
    ip_address = log_line[:line_index] # Check to make sure it is valid. Do not
    #                                    enter into database unless it starts with
    #                                    an IP address.
    if L2Dcfg.DEBUG: print(f"IP Address: {ip_address}")

    # Parse the remote logname field. ----------
    # Remote logname (from identd, if supplied). This will return a dash unless
    # mod_ident is present and IdentityCheck is set On.
    last_index = line_index
    line_index = log_line.find(' ',line_index+1)
    remote_logname = log_line[last_index:line_index]
    if L2Dcfg.DEBUG: print(f"Remote Logname: {remote_logname}")

    # Parse the remote logname field. ----------
    # Remote user if the request was authenticated. May be bogus if
    # return status (%s) is 401 (unauthorized).
    last_index = line_index
    line_index = log_line.find(' ',line_index+1)
    remote_user = log_line[last_index:line_index]
    if L2Dcfg.DEBUG: print(f"Remote User: {remote_user}")

    # Parse the request time field. ----------
    # Time the request was received, in the format [18/Sep/2011:19:18:28 -0400].
    # The last number indicates the timezone offset from GMT
    # NOTE: Break this down further into day month year hour minute second
    last_index = log_line.find('[',line_index+1)+1
    line_index = log_line.find(']',last_index)
    time_stamp = log_line[last_index:line_index]
    if L2Dcfg.DEBUG: print(f"Time Stamp: {time_stamp}")

    # Parse the request field. ----------
    # First line of request.
    # NOTE: Check that we are actually at the end of the request. The request
    #       terminates with a '" [2-4][01][0-9] ' character set.
    last_index = log_line.find(' "',line_index)+2
    pattern = r"\" \d{3} "
    test_str = log_line[last_index:]
    if L2Dcfg.DEBUG: print(f"Test String: {test_str}")
    match = re.split(pattern,test_str)
    if L2Dcfg.DEBUG: print(f"Match: {match}")
    request = match[0]
    # Generate correct line index since we use different method here.
    line_index = last_index+len(request)
    if L2Dcfg.DEBUG: print(f"Request: {request}")

    # Parse the final status code field. ----------
    # Status. For requests that have been internally redirected, this is the
    # status of the original request. Use %>s for the final status.
    last_index = line_index+2
    line_index = log_line.find(' ',last_index)
    status_code = log_line[last_index:line_index]
    if L2Dcfg.DEBUG: print(f"Final Status Code: {status_code}")

    # Parse the bytes sent field. ----------
    # Bytes sent, including headers. May be zero in rare cases such as when a
    # request is aborted before a response is sent. You need to enable mod_logio
    # to use this.
    last_index = line_index+1
    line_index = log_line.find(' ',last_index)
    bytes_sent = log_line[last_index:line_index]
    if L2Dcfg.DEBUG: print(f"Bytes Sent: {bytes_sent}")

    # Parse the referer field. ----------
    # The contents of VARNAME: header line(s) in the request sent to the server.
    last_index = log_line.find(' "',line_index)
    line_index = log_line.find('" ',last_index)
    referer = log_line[(last_index+2):line_index]
    if L2Dcfg.DEBUG: print(f"Referer: {referer}")

    # Parse the user agent field. ----------
    # The contents of VARNAME: header line(s) in the request sent to the server.
    last_index = log_line.find(' "',line_index)
    line_index = log_line.find('" ',last_index)
    end_index = len(log_line)-1
    if end_index > last_index+18:
        user_agent = log_line[(last_index+2):end_index]
        forensic_id = "-"
    else:
        user_agent = log_line[(last_index+2):line_index]
        if L2Dcfg.DEBUG:
            print(f"User Agent: {user_agent}")
            print({last_index},{line_index})

        # Parse the forensic ID field. ----------
        # The contents of note VARNAME from another module.
        # NOTE: Only used starting 2023 logs, so intelligence is necessary to
        # interpret.
        if line_index > 0: # Get forensic_used into a static class.
          last_index = line_index
          line_index = log_line.find(' ',last_index)+1
          if L2Dcfg.DEBUG: print({last_index},{line_index},{end_index})
          forensic_id = log_line[line_index:end_index]
          if L2Dcfg.DEBUG: print(f"Forensic ID: {forensic_id}")
        else: forensic_id = "-"

    if L2Dcfg.DEBUG:
        return pause_until_key_press()

    # Configure array to return to caller.
    result = [ip_address, remote_logname, remote_user, time_stamp, request, status_code, bytes_sent, referer, user_agent, forensic_id]
    return result
